@@ -8,6 +8,7 @@ import warnings
 
 class _Model:
     """Abstract class for models"""
+    # TODO init model by only name to be able to load from previous saved instance
 
     def __init__(self, A:'initial_vibration'=None, alpha: 'instance of Alpha class'=None,
                  conditions: 'list of condition instances'=None, name:'string'=''):
@@ -20,7 +21,11 @@ class _Model:
         name: optional name of the model -> string
         """
         self.name = name
-        if conditions is None:
+        self.alpha = alpha
+        self.W = None
+        self.split_instance = []  # List of all related splits that has modified the solution
+        self.conditions = conditions
+        if self.conditions is None:
             if A is None or alpha is None:
                 raise TypeError('Either (A and Alpha) or `conditions` should be assigned.')
             try:
@@ -32,18 +37,18 @@ class _Model:
             except IndexError:
                 raise tools.CustomError('`A` should be a column vector (Mx1) dimension')
 
-            self.split_instance = []  # List of all related splits that has modified the solution
             # Test if Alpha is an instance of Alpha class
             if not isinstance(alpha, Alpha):
                 raise tools.CustomError('Please create an `Alpha instance` first --> ex. alpha = Alpha()')
             else:
+                self.alpha = alpha
                 self.ALPHA = alpha.value
-        elif isinstance(conditions, list) == False:
+        elif isinstance(self.conditions, list) == False:
             raise TypeError('Conditions should be a list')
         else:
-            if all(isinstance(condition, Condition) for condition in conditions):
-                self.ALPHA = np.vstack([condition.alpha.value for condition in conditions])
-                self.A = np.vstack([condition.A for condition in conditions])
+            if all(isinstance(condition, Condition) for condition in self.conditions):
+                self.ALPHA = np.vstack([condition.alpha.value for condition in self.conditions])
+                self.A = np.vstack([condition.A for condition in self.conditions])
             else:
                 raise TypeError('''`conditions` should be a list of `Condition class` try condition
                                 = Condition()''')
@@ -188,6 +193,8 @@ class _Model:
             _S = np.array(np.round(_S.value))
             self._S = _S
             self._prob_S = _prob_S.value
+            _W_equ_split = np.sum(self.weight_available_matrix * self._S)
+            self._W_equ_split = _W_equ_split
             return _S
 
 
@@ -195,7 +202,7 @@ class _Model:
         def results(self):
             '''
             Method to print comprehensive results of the solution
-            transfer the numpy array to non-zeros dataframe
+            transfer the numpy array to non-zeros DataFrame
             Args:
                 Instance of split model
             Return:
@@ -203,6 +210,7 @@ class _Model:
             '''
             df_splitting = pd.DataFrame(self._S, index=self.weights_available, columns=self.holes_available)
             df_splitting = df_splitting.loc[:, (df_splitting != 0).any(axis=0)]
+            df_splitting.index.name = 'weights_available'
             return df_splitting
         def error(self, options='error'):
             '''
@@ -217,8 +225,6 @@ class _Model:
             '''
 
             # Error equivilant weight after splitting
-            _W_equ_split = np.sum(self.weight_available_matrix * self._S)
-            self._W_equ_split = _W_equ_split
             _error = abs(self._W_equ_split - self.weight)
             # Estimate the error generated by using minmax instead of norm2 in the objective function
             _problem_error = (self._prob_S - _error) / abs(self.weight)
@@ -227,7 +233,7 @@ class _Model:
             elif options== 'problem_error':
                 return _problem_error
             elif options == 'equ':
-                return tools.convert_cart_math(_W_equ_split)
+                return tools.convert_cart_math(self._W_equ_split)
             else:
                 raise tools.CustomError('Invalid options. Choose options="error" for solution error'
                                         'caused by splitting, options="problem_error" for solution'
@@ -243,7 +249,7 @@ class _Model:
                             estimate the RMSE and residual_vibration change caused by splitting
 
             '''
-            if confirm:
+            if confirm and self._W_equ_split is not None:
                 self.model.W[self.balancing_plane_index] = self._W_equ_split
                 self.model.split_instance.append(self)
             elif not confirm:
@@ -252,6 +258,40 @@ class _Model:
                 self.model.W[self.balancing_plane_index] = self.weight
             return self.model.W
 
+    def _info(self):
+        '''
+        Method to summarize the results for Model.
+        '''
+        yield ('MODEL TYPE', type(self).__name__)
+        if self.name:
+            yield ('MODEL NAME' , self.name)
+        if self.alpha is not None:
+            yield ('INFLUENCE COEFFICIENT MATRIX', str(self.alpha))
+        if self.A is not None and self.conditions is None:
+            _index = (f'Sensor {m+1}' for m in range(self.A.shape[0]))
+            yield ('INITIAL VIBRATION' , pd.DataFrame(tools.convert_cart_math(self.A),
+                        index=_index, columns=['Vibration']))
+        if self.conditions is not None:
+            yield ('CONDITIONS',''.join(str(condition) for condition in self.conditions))
+        if self.W is not None:
+            _index = (f'Plane {n+1}' for n in range(self.W.shape[0]))
+            yield ('SOLUTION', pd.DataFrame(tools.convert_cart_math(self.W), index=_index,
+                        columns=['Correction Masses']))
+            yield ('RMSE', self.rmse())
+            _index = (f'Sensor {m+1}' for m in range(self.A.shape[0]))
+            yield ('Resiudal Vibration Expected', pd.DataFrame(tools.convert_cart_math(self.expected_residual_vibration()), index=_index,
+                        columns=['Expected Vibration']))
+        else:
+            yield ('SOLUTION','No solution calculated.')
+
+        if self.split_instance is not None :
+            if self.split_instance:
+                yield ('SPLITS','\n\n'.join(str(split.results()) for split in self.split_instance))
+    def info(self):
+        formatter = tools.InfoFormatter(name='MODEL', info_parameters=self._info(), level=3)
+        return ''.join(formatter.info())
+
+
 
 class LeastSquares(_Model):
     """subclass of Model
@@ -303,7 +343,7 @@ class Min_max(_Model):
     to minimize the maximum of residual_vibration.
     """
 
-    def __init__(self, A:np.array, alpha:'instance of Alpha class', weight_const={}, name=''):
+    def __init__(self, A:np.array, alpha:'instance of Alpha class', conditions=None, weight_const={}, name=''):
         """ Instantiate the model
         Args:
             A: Initial vibration vector -> np.ndarray
@@ -312,7 +352,7 @@ class Min_max(_Model):
         Returns: solution matrix W
         """
         self.weight_const = weight_const
-        super().__init__(A, alpha, name=name)
+        super().__init__(A=A, alpha=alpha, conditions=conditions, name=name)
 
     def solve(self, solver=None):
         '''
@@ -344,6 +384,7 @@ class LMI(_Model):
     """
 
     def __init__(self, A:np.array, alpha:'instance of Alpha class',
+                 conditions=None,
                  weight_const={},
                  critical_planes={}, V_max=None, name=''):
         """
@@ -358,7 +399,7 @@ class LMI(_Model):
         self.weight_const = weight_const
         self.critical_planes = list(critical_planes)
         self.V_max = V_max
-        super().__init__(A, alpha, name=name)
+        super().__init__(A=A, alpha=alpha, conditions= conditions, name=name)
 
     def solve(self, solver=None):
         '''
@@ -442,3 +483,5 @@ class LMI(_Model):
         W = _WR + _WI * 1j
         self.W = W.value
         return self.W
+
+
